@@ -1,7 +1,7 @@
 #!/bin/sh
 # Verify that ls -i and find -printf %i produce the same output.
 
-# Copyright (C) 2011-2019 Free Software Foundation, Inc.
+# Copyright (C) 2011-2021 Free Software Foundation, Inc.
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,11 +16,15 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
-. "${srcdir=.}/tests/init.sh"
+. "${srcdir=.}/tests/init.sh"; fu_path_prepend_
 print_ver_ find oldfind
 
 make_canonical() {
-  sed -e 's/ /_/g'
+  sed -e '
+    # Solaris ls outputs with leading padding blanks; strip them.
+    s/^ *//g;
+    # Squeeze blanks between inode number and name to one underscore.
+    s/ /_/g'
 }
 
 # Create a file.
